@@ -42,6 +42,8 @@ void loadConfig(void)
 	FILE *f;
 
 	// set default config values first
+	config.noDownsampleOnSmpLoad = false;
+	config.disableE8xEffect = false;
 	config.fullScreenStretch = false;
 	config.pattDots = false;
 	config.waveformCenterLine = true;
@@ -66,6 +68,9 @@ void loadConfig(void)
 	config.integerScaling = true;
 	config.audioInputFrequency = 44100;
 
+	config.maxSampleLength = 65534;
+	config.reservedSampleOffset = (MOD_SAMPLES+1) * config.maxSampleLength;
+
 #ifndef _WIN32
 	getcwd(oldCwd, PATH_MAX);
 #endif
@@ -192,6 +197,35 @@ static bool loadProTrackerDotIni(FILE *f)
 			continue;
 		}
 
+		// 64K_LIMIT
+		else if (!_strnicmp(configLine, "64K_LIMIT=", 10))
+		{
+			if (!_strnicmp(&configLine[10], "TRUE", 4))
+			{
+				config.maxSampleLength = 65534;
+				config.reservedSampleOffset = (MOD_SAMPLES+1) * config.maxSampleLength;
+			}
+			else if (!_strnicmp(&configLine[10], "FALSE", 5))
+			{
+				config.maxSampleLength = 131070;
+				config.reservedSampleOffset = (MOD_SAMPLES+1) * config.maxSampleLength;
+			}
+		}
+
+		// NO_DWNSMP_ON_SMP_LOAD (no dialog for 2x downsample after >22kHz sample load)
+		else if (!_strnicmp(configLine, "NO_DWNSMP_ON_SMP_LOAD=", 22))
+		{
+			if (!_strnicmp(&configLine[22], "TRUE", 4)) config.noDownsampleOnSmpLoad = true;
+			else if (!_strnicmp(&configLine[22], "FALSE", 5)) config.noDownsampleOnSmpLoad = false;
+		}
+
+		// DISABLE_E8X (Karplus-Strong command)
+		else if (!_strnicmp(configLine, "DISABLE_E8X=", 12))
+		{
+			     if (!_strnicmp(&configLine[12], "TRUE",  4)) config.disableE8xEffect = true;
+			else if (!_strnicmp(&configLine[12], "FALSE", 5)) config.disableE8xEffect = false;
+		}
+
 		// HWMOUSE
 		else if (!_strnicmp(configLine, "HWMOUSE=", 8))
 		{
