@@ -15,11 +15,12 @@ typedef struct config_t
 	char *defModulesDir, *defSamplesDir;
 	bool waveformCenterLine, pattDots, compoMode, autoCloseDiskOp, hideDiskOpDates, hwMouse;
 	bool transDel, fullScreenStretch, vsyncOff, modDot, blankZeroFlag, realVuMeters, rememberPlayMode;
-	bool startInFullscreen, integerScaling;
+	bool startInFullscreen, integerScaling, disableE8xEffect, noDownsampleOnSmpLoad;
 	int8_t stereoSeparation, videoScaleFactor, accidental;
 	uint8_t pixelFilter, filterModel;
 	uint16_t quantizeValue;
-	uint32_t soundFrequency, soundBufferSize, audioInputFrequency;
+	int32_t maxSampleLength;
+	uint32_t soundFrequency, soundBufferSize, audioInputFrequency, reservedSampleOffset;
 } config_t;
 
 extern config_t config; // pt2_config.c
